.include "nes.inc"

out_buffer = $0100

.export bagel_demo

.importzp temp, rng_seed
.importzp bagel_stream_ptr, ntd_stream_ptr, pkb_stream_ptr, bagel_block_count

.import rng_stepBit, rng_stepByte, rng_init
.import bagel_decompress_block
.import bagel_bulk_upload
;.import bagel_decompress_128_bytes
.import ntd_decode
.import pads_read, pads_rejectOpposingDirections
.import pkb_decode

.segment "ZEROPAGE": zeropage
  ppu_ctrl_mirror:  .res 1
  nmi_stash:        .res 1
  nmi_count:        .res 1
  ppu_stream_block_rdy:   .res 1
  stream_ppuaddr_bank1:   .res 2
  stream_ppuaddr_bank2:   .res 2
  reset_stream_ptr: .res 2
  cur_keys:         .res 4
;  prev_keys_inv:    .res 4
  new_keys:         .res 4
  any_new_keys:     .res 1
  game_target_number:     .res 3
  game_current_number:    .res 3
  game_cursor_pos:        .res 1
  game_cursor_display_offset:    .res 1
  game_cursor_display_phase:     .res 1

stream_ptr = bagel_stream_ptr

.segment "CODE"

bagel_decompress_128_bytes:
rts

bagel_demo:
RST:
  sei                   ; Disable interrupts
  ldx #%00000000
  stx PPU_CTRL          ; Disable NMI and set VRAM increment to 1
  stx PPU_MASK          ; Disable rendering
  stx APU_DMC_FREQ      ; Disable DMC IRQ
  bit APU_STATUS        ; Acknowledge DMC IRQ
  lda #%01000000
  sta APU_FRAMECNT      ; Disable APU Frame IRQ
  lda #%00001111
  sta APU_SND_CHN       ; Disable DMC playback, initialize other channels
  cld                   ; Post-patent famiclone compatibility.
  dex
  txs
  bit PPU_STATUS        ; Acknowledge stray vblank NMI across reset
  vwait1:
    bit PPU_STATUS
  bpl vwait1
  jsr rng_init
  vwait2:
    bit PPU_STATUS
  bpl vwait2

  ldx #4-1
  clear_zp_loop:
    lda #$ff
    sta cur_keys, x
    lda #$00
    sta game_current_number, x
    dex
  bpl clear_zp_loop

  lda #<test_stream
  sta stream_ptr+0
  lda #>test_stream
  sta stream_ptr+1
  jsr example_screen_compositor

  lda #<(compressed_chr+1)
  sta reset_stream_ptr+0
  lda #>(compressed_chr+1)
  sta reset_stream_ptr+1

  lda #VBLANK_NMI|BG_0000|OBJ_0000|NT_2000
  sta ppu_ctrl_mirror
  lda #$00
  sta ppu_stream_block_rdy

main_loop:
  jsr wait_for_vblank

  lda ppu_stream_block_rdy
  beq skip_ppu_stream_upload
    tsx
    stx nmi_stash
      ldx #$ff
      txs
      
      lda stream_ppuaddr_bank1+1
      sta PPU_ADDR
      lda stream_ppuaddr_bank1+0
      sta PPU_ADDR
      
      ldx #(64/4)
      upload_block1_loop:
        .repeat 4
          pla
          sta PPU_DATA
        .endrepeat
        dex
      bne upload_block1_loop

      lda stream_ppuaddr_bank2+1
      sta PPU_ADDR
      lda stream_ppuaddr_bank2+0
      sta PPU_ADDR
      
      ldx #(64/4)
      upload_block2_loop:
        .repeat 4
          pla
          sta PPU_DATA
        .endrepeat
        dex
      bne upload_block2_loop
      
      lda #0
      sta ppu_stream_block_rdy
    ldx nmi_stash
    txs
  skip_ppu_stream_upload:

  lda #$00
  sta PPU_SCROLL
  sta PPU_SCROLL
  lda ppu_ctrl_mirror
  sta PPU_CTRL
  lda #BG_ON|OBJ_OFF
  sta PPU_MASK

  wait_for_button_press:
    jsr rng_stepBit
    jsr read_new_buttons
    lda bagel_block_count
    bne end_button_press_loop
    lda any_new_keys
  beq wait_for_button_press
  end_button_press_loop:

  lda new_keys
  and #BUTTON_A
  beq dont_swap_chr_bank
    lda ppu_ctrl_mirror
    eor #BG_1000
    sta ppu_ctrl_mirror
  dont_swap_chr_bank:

  lda new_keys
  and #BUTTON_B
  beq dont_reset_stream
    lda #BG_OFF|OBJ_OFF
    sta PPU_MASK
    
    lda #$00
    sta PPU_ADDR
    sta PPU_ADDR
    tax
    ldy #$100-$20
    clear_nt_outer_loop:
      clear_nt_inner_loop:
        sta PPU_DATA
        inx
      bne clear_nt_inner_loop
      iny
    bne clear_nt_outer_loop
    jmp init_stream
  dont_reset_stream:

  lda cur_keys
  and #BUTTON_B
  beq glitch_ptr_test
    lda bagel_block_count
    cmp #$80
    bcs glitch_ptr_test2
      lda cur_keys
      and #BUTTON_LEFT
      beq glitch_ptr_test_left
        lda reset_stream_ptr+0
        bne @__skip1
          dec reset_stream_ptr+1
        @__skip1:
        dec reset_stream_ptr+0
        jmp init_stream
      glitch_ptr_test_left:

      lda cur_keys
      and #BUTTON_RIGHT
      beq glitch_ptr_test_right
        inc reset_stream_ptr+0
        bne @__skip2
          inc reset_stream_ptr+1
        @__skip2:
        jmp init_stream
      glitch_ptr_test_right:
    glitch_ptr_test2:
  glitch_ptr_test:

  jmp gotos_are_harmful
  init_stream:
    lda reset_stream_ptr+0
    sta stream_ptr+0
    lda reset_stream_ptr+1
    sta stream_ptr+1
    lda #0
    sta ppu_stream_block_rdy
    lda #(8192/64)
    sta bagel_block_count
    lda #<($10000-64)
    sta stream_ppuaddr_bank1+0
    lda #>($10000-64)
    sta stream_ppuaddr_bank1+1
    lda #<($1000-64)
    sta stream_ppuaddr_bank2+0
    lda #>($1000-64)
    sta stream_ppuaddr_bank2+1
  gotos_are_harmful:

  lda bagel_block_count
  beq no_blocks_to_go
    jsr bagel_decompress_128_bytes
    clc
    lda stream_ppuaddr_bank1+0
    adc #64
    sta stream_ppuaddr_bank1+0
    lda stream_ppuaddr_bank1+1
    adc #0
    sta stream_ppuaddr_bank1+1

    clc
    lda stream_ppuaddr_bank2+0
    adc #64
    sta stream_ppuaddr_bank2+0
    lda stream_ppuaddr_bank2+1
    adc #0
    sta stream_ppuaddr_bank2+1

    lda #1
    sta ppu_stream_block_rdy
  no_blocks_to_go:

  lda new_keys
  and #BUTTON_START
  beq dont_start_game
    jmp start_game
  dont_start_game:
jmp main_loop

start_game:
  jsr wait_for_vblank
  lda #BG_OFF|OBJ_OFF
  sta PPU_MASK

  ldx #3-1
  generate_target_number_loop:
    try_rng_again:
      jsr rng_stepByte
      and #$0f
      cmp #10
    bcs try_rng_again
    sta game_target_number, x
    dex
  bpl generate_target_number_loop

  lda #<compressed_game_stream
  sta stream_ptr+0
  lda #>compressed_game_stream
  sta stream_ptr+1
  jsr example_screen_compositor

  ldx #97-1
  copy_ppu_strings_loop:
    lda game_ppu_strings, x
    sta out_buffer, x
    dex
  bpl copy_ppu_strings_loop

  lda #1
  sta game_cursor_pos
  lda #0
  sta temp+6
  sta game_cursor_display_phase

game_loop:
  jsr wait_for_vblank

draw_ppu_strings:
  tsx
  stx nmi_stash
    ldx #$ff
    string_loop:
      pla
      bmi end_ppu_strings
      sta PPU_ADDR
      ldy #VBLANK_NMI
      asl
      bpl not_down
        ldy #VBLANK_NMI|VRAM_DOWN
      not_down:
      pla
      sta PPU_ADDR
      sty PPU_CTRL
      pla
      tay
      bmi do_run
        lit_loop:
          pla
          sta PPU_DATA
          dey
        bpl lit_loop
      bmi string_loop
      do_run:
        pla
        run_loop:
          sta PPU_DATA
          iny
        bne run_loop
      beq string_loop
    end_ppu_strings:
  ldx nmi_stash
  txs

  lda #$00
  sta PPU_SCROLL
  sta PPU_SCROLL
  lda #VBLANK_NMI|BG_1000|OBJ_0000|NT_2400
  sta PPU_CTRL
  lda #BG_ON|OBJ_OFF
  sta PPU_MASK

game_logic:
  jsr read_new_buttons

  lda temp+6
  cmp #3
  bcc not_win_yet
    ldx #18-1
    copy_win_string_loop:
      lda game_you_win_string, x
      sta $014c, x
      dex
    bpl copy_win_string_loop
    bmi game_loop
  not_win_yet:

  lda game_cursor_pos
  asl
  clc
  adc game_cursor_pos
  clc
  adc #$cc
  sta game_cursor_display_offset

  lda game_cursor_display_phase
  beq display_movement_done
    clc
    adc game_cursor_display_offset
    sta game_cursor_display_offset
    lda #0
    sta game_cursor_display_phase
  display_movement_done:

  ldx game_cursor_pos
  ldy game_cursor_pos

  lda new_keys
  and #BUTTON_LEFT
  beq game_key_left
    dex
    cpx #$100-1
    bne no_warp_left
      ldx #2
    no_warp_left:
    dec game_cursor_display_offset
    lda #1
    sta game_cursor_display_phase
  game_key_left:

  lda new_keys
  and #BUTTON_RIGHT
  beq game_key_right
    inx
    cpx #3
    bne no_warp_right
      ldx #0
    no_warp_right:
    inc game_cursor_display_offset
    lda #$100-1
    sta game_cursor_display_phase
  game_key_right:

  lda cur_keys
  and #BUTTON_B
  beq game_swap
    lda game_current_number, x
    pha
      lda game_current_number, y
      sta game_current_number, x
    pla
    sta game_current_number, y
  game_swap:

  stx game_cursor_pos

  ldy game_current_number, x

  lda new_keys
  and #BUTTON_UP
  beq game_key_up
    iny
    cpy #10
    bne no_warp_up
      ldy #0
    no_warp_up:
  game_key_up:

  lda new_keys
  and #BUTTON_DOWN
  beq game_key_down
    dey
    cpy #$100-1
    bne no_warp_down
      ldy #9
    no_warp_down:
  game_key_down:

  sty game_current_number, x

  lda new_keys
  and #BUTTON_A|BUTTON_START
  beq game_key_enter
    ldy $0119
    ldx $011a
    lda $011b
    inx
    cpx #'0'+10
    bcc digit_2
      ldx #'0'
      iny
      cpy #'0'+10
      bcc digit_3
        ldy #'9'
        ldx #'9'
        lda #'+'
      digit_3:
    digit_2:
    sty $0119
    stx $011a
    sta $011b

    lda #0
    sta temp+4
    sta temp+5
    sta temp+6
    ldy #3-1
    gen_hints_loop:
      lda #0
      sta temp+7
      lda game_current_number, y
      ldx #3-1
      pico_detect_loop:
        cmp game_target_number, x
        bne not_pico
          inc temp+7
          bne break_pico_detect_loop  ;,; jmp
        not_pico:
        dex
      bpl pico_detect_loop
      break_pico_detect_loop:
      cmp game_target_number, y
      bne not_fermi
        inc temp+7
      not_fermi:
      ldx temp+7
      inc temp+4, x
      dey
    bpl gen_hints_loop
    ldx #3-1
    draw_hints_loop:
      lda #' '
      cpx temp+4
      bcs max_bagel_count
        lda #$dd
      max_bagel_count:
      sta $0103, x
      lda #' '
      cpx temp+5
      bcs max_pico_count
        lda #$dd
      max_pico_count:
      sta $0109, x
      lda #' '
      cpx temp+6
      bcs max_fermi_count
        lda #$dd
      max_fermi_count:
      sta $0113, x
      dex
    bpl draw_hints_loop
  game_key_enter:

game_render:
  ldy #$23
  jsr render_thingy
  ldy #$33
  jsr render_thingy
  ldy #$43
  jsr render_thingy

  ldx game_cursor_display_offset
  stx $0157
  inx
  stx $015c
  txa
  clc
  adc #$20-1
  tax
  stx $014d
  inx
  stx $0152

jmp game_loop


NMI:
  inc nmi_count
IRQ:
rti

wait_for_vblank:
  lda nmi_count
  vwait3:
    cmp nmi_count
  beq vwait3
rts


render_thingy:
  ldx #0
  jsr @render_thingy_row
  inx
  iny
  jsr @render_thingy_row
  inx
  iny
@render_thingy_row:
  jsr @render_thingy_byte
@render_thingy_byte:
  lda $0100, y
  and #$f0
  ora game_current_number, x
  sta $0100, y
  iny
rts


read_new_buttons:
  jsr pads_read
  jsr pads_rejectOpposingDirections

  lda #0
  sta any_new_keys
  ldx #4-1
  copy_keys_loop:
    lda cur_keys, x
    eor #$ff
;    sta prev_keys_inv, x
    and temp+0, x
    sta new_keys, x
    ora any_new_keys
    sta any_new_keys
    lda temp+0, x
    sta cur_keys, x
    dex
  bpl copy_keys_loop
rts


example_screen_compositor:
  jsr stream_read_byte
  tax
  jsr stream_read_byte
  cmp #$ff
  beq end_stream
    sta PPU_ADDR
    stx PPU_ADDR
    ; it's ok to set PPU_CTRL after PPU_ADDR when rendering is off
    ; because the PPU 'v' register is not changed by PPU_CTRL.
    asl
    bpl dont_draw_down
      ldy #VBLANK_NMI|VRAM_DOWN
      sty PPU_CTRL
      jsr pkb_decode
    jmp example_screen_compositor
    dont_draw_down:
    ldy #VBLANK_NMI
    sty PPU_CTRL
    bcs do_bagel_bulk_upload
      jsr ntd_decode
    jmp example_screen_compositor
    do_bagel_bulk_upload:
      jsr bagel_bulk_upload
    jmp example_screen_compositor
  end_stream:
rts

stream_read_byte:
  ldy #$00
  lda (stream_ptr), y
  inc stream_ptr+0
  bne @__skip0
    inc stream_ptr+1
  @__skip0:
rts


.segment "RODATA"
test_stream:
; The palette
.word $3fe0
  .byte $20-16 | %10000000
  .byte   $0f,$00,$10,$20
  .byte   $0f,$0f,$2d,$30
  .byte   $0f,$0c,$1c,$3c
  .byte   $0f,$05,$15,$35
  .byte $20-16 | %10100000, $0f, $30
  .byte $ff

.word $2000
  .incbin "src/screens/bagel-screens.nam.ntd"

.word $20c7 | $4000
  .byte 257-16, $85
  .byte $80
.word $20d8 | $4000
  .byte 257-16, $8a
  .byte $80
.word $24c7 | $4000
  .byte 257-16, $dc
  .byte $80
.word $24d8 | $4000
  .byte 257-16, $de
  .byte $80

.word $0000 | $8000
compressed_chr:
  .incbin "obj/bagel-demo.chr.bagel"
.word $0000 | $ffff


compressed_game_stream:
.word $1a00 | $8000
compressed_game_chr:
  .incbin "obj/bagel-game.chr.bagel"
.word $0000 | $ffff

; In the future I could try to make a compositor with box drawing features.
; and that can work with a series of PPU strings in this form:
; Dest:
;   0Dyyyyyy xxxxxxxx : Set PPU_CTRL 32-INC mode to D
;                       and PPU_ADDR to yyxx
;   1-------          : end list of strings.
; length and type:
;   0nnnnnnn ...      : N+1 literal bytes.
;   1nnnnnnn xx       : 128-N run of X.
game_ppu_strings:
.byte $26, $32
  .byte 3-1, "   "
.byte $26, $52
  .byte 3-1, "   "
.byte $25, $eb
  .byte $100-10, " "
.byte $26, $72
  .byte 3-1, "   "
.byte $26, $92
  .byte 3-1, "00 "
.byte $25, $cb
  .byte $100-10, " "
.byte $25, $4c
  .byte 9-1, $a0, $d0, $20, $a0, $d0, $20, $a0, $d0, $20
.byte $25, $0b
  .byte $100-10, " "
.byte $25, $6c
  .byte 9-1, $b0, $e0, $20, $b0, $e0, $20, $b0, $e0, $20
.byte $24, $eb
  .byte $100-10, " "
.byte $25, $8c
  .byte 9-1, $c0, $f0, $20, $c0, $f0, $20, $c0, $f0, $20

.byte $24 | $40, $ef
  .byte 2-1, $aa, $ba
.byte $24 | $40, $f0
  .byte 2-1, $da, $ea
.byte $25 | $40, $cf
  .byte 2-1, $bb, $cb
.byte $25 | $40, $d0
  .byte 2-1, $eb, $fb

.byte $ff

game_you_win_string:
.byte $25, $c9
  .byte 14-1, "Correct guess!"
.byte $ff

