.include "nes.inc"

.export cart_vectors

.import init_reset_handler
.exportzp temp
.exportzp nmi_count, tv_type, tvSystem, nmi_stash, rng_seed
.exportzp ramvectors_nmi, ramvectors_irq
.exportzp ramvectors_nmi_opcode, ramvectors_irq_opcode
.exportzp ramvectors_nmi_addr, ramvectors_irq_addr

.segment "INESHDR"
INES_MAPPER     = 34    ; 0 = NROM, 34 = BNROM
INES_PRG_COUNT  = 1     ; PRG ROM size in 16384 byte units
INES_CHR_COUNT  = 0     ; CHR ROM size in 8192 byte units
INES_MIRRORING  = 1     ; 0 = horizontal, 1 = vertical, 8 = four-screen
INES_SRAM       = 0     ; 1 = battery backed SRAM at $6000-7FFF
  .byte "NES",$1A       ; magic signature
  .byte INES_PRG_COUNT
  .byte INES_CHR_COUNT
  .byte ((INES_MAPPER & $0f) << 4) | (INES_SRAM << 1) | INES_MIRRORING
  .byte (INES_MAPPER & $f0)
  .byte 0,0,0,0,0,0,0,0     ; no other special features

.segment "ZEROPAGE": zeropage
  temp:                     .res 16

.segment "SYSTEMZP": zeropage
  rng_seed:                 .res 4
    ; so that the partial zeropage clear will not mess with RNG seeding
  nmi_stash:                .res 1
  _unused:                  .res 3
  tvSystem:                         ; pently alias
  tv_type:                  .res 1
  ramvectors_nmi:
    ramvectors_nmi_opcode:  .res 1
    ramvectors_nmi_addr:    .res 2
  nmi_count:                .res 1
  ramvectors_irq:
    ramvectors_irq_opcode:  .res 1
    ramvectors_irq_addr:    .res 2

.segment "TRAMPOLINE"

.segment "VECTORS"
NMI = ramvectors_nmi
IRQ = ramvectors_irq
RST = init_reset_handler

cart_title:
; 16 byte zero filled UTF-8 title.
  .byte "Controller Test", $00
cart_bank_checksum:
; the checksum is the negated result of adding together
; all little endian words xor with 0xffff in $8000-$ffff
  .word 65535   ; computed by external tool.
cart_reset_stub:
  lda #(15 | %11110000)
  sta cart_reset_stub+1
jmp RST
cart_vectors:
  .addr NMI, cart_reset_stub, IRQ

