;   Draw controller 2 input onto the NES screen.
;   Copyright (C) 2016  Johnathan Roatch
;
;   Copying and distribution of this file, with or without
;   modification, are permitted in any medium without royalty
;   provided the copyright notice and this notice are preserved
;   in all source code copies.
;
;   This file is offered as-is, without any warranty.

.include "nes.inc"

.importzp temp
.importzp pad_1_keys, pad_1_axlr, pad_2_keys, pad_2_axlr
;.import ppu_oam

.import pads_read_16bit, bagel_bulk_upload, ppu_set_addr, ppu_vsync, ppu_forced_blank
.export button_logger_activity

.import easter_egg_entry

.segment "CODE"

.proc button_logger_activity
update_counter = temp+1
address_ptr_lo = temp+2
address_ptr_hi = temp+3
stop_state = temp+4
sliver_pattren = temp+5

boot:
  sei                   ; Disable IRQ
  ldx #%00000000
  stx PPU_CTRL          ; Disable NMI
    ; Stop IRQ and NMI interrupts ASAP.

; Putting other sources of interrupts into a known state.
  stx PPU_MASK          ; Disable rendering
  stx APU_DMC_FREQ      ; Disable DMC IRQ
  dex  ;,; ldx #$ff
  txs
  bit APU_STATUS        ; Acknowledge DMC IRQ
  lda #%01000000
  sta APU_FRAMECNT      ; Disable APU Frame IRQ
  lda #%00001111
  sta APU_SND_CHN       ; Disable DMC playback, initialize other channels
  cld                   ; Turn off decimal mode for compatibility.

; tv_type var not needed for button logger.

; Acknowledge stray vblank NMI across reset
  bit PPU_STATUS
; It takes one full frame for the PPU to become stable.
; Best practice is to spin twice until the vblank flag of
; PPU_STATUS is set.
  vwait1:
    bit PPU_STATUS
  bpl vwait1

  inx  ;,; ldx #$00
  txa  ;,; lda #0
  clear_zp_loop:
    dex
    sta 0, x
  bne clear_zp_loop

  vwait2:
    bit PPU_STATUS
  bpl vwait2
  
;  cli

  jsr ppu_forced_blank

init:
  lda #$3f
  ldy #$e0
  jsr ppu_set_addr

  lda #%11001010
  ldx #$0f
  ldy #$e0
  jsr do_ppu_pattren_load

  lda #$0f
  ldy #$00
  jsr ppu_set_addr

  lda #>font
  ldy #<font
  jsr bagel_bulk_upload

  lda #$2c
  ldy #$00
  jsr ppu_set_addr

  ldy #$100-130
  jsr write_minus_y_number_of_blanks

  ;,; ldy #$00
  sty temp+0
  nt_data_loop:
    jsr draw_log_row
    jsr draw_log_row
    ldy #$100-4
    jsr write_minus_y_number_of_blanks
    stx temp+0
    cpx #21*12
  bne nt_data_loop

  ldy #$100-158
  jsr write_minus_y_number_of_blanks

  lda #%11110000
  ldx #$00
  ldy #$100-64
  jsr do_ppu_pattren_load

  tya ;,; lda #0
  ldx #6
  clear_temp_loop:
    dex
    sta temp, x
  bne clear_temp_loop
  lda #$03
  sta sliver_pattren

  lda #VBLANK_NMI|NT_2C00
  sta PPU_CTRL
main_loop:
  jsr ppu_vsync
;  lda #>(ppu_oam)
;  sta OAM_DMA
  jsr pads_read_16bit
  lda JOY_PORT_2
  and #$18
  eor pad_2_axlr
  sta pad_2_axlr
  sta pad_1_axlr
  ;,; lda PPU_STATUS

  lda pad_1_keys
  and stop_state
  and #BUTTON_A|BUTTON_B
  lsr
  lsr
  eor stop_state
  cmp #$a0
  bne alt_tigger_exit
    eor #$30
  alt_tigger_exit:
  cmp #$50
  beq fix_a_continuing
  cmp #$f0
  bne trigger_not_set
    ldx pad_2_keys
    beq not_triggered
fix_a_continuing:
      and #$c0
    not_triggered:
  trigger_not_set:
  sta stop_state

  ;,; lda stop_state
  lsr
  ora pad_1_keys
  and #BUTTON_START
  bne paused
    lda #12
    sta update_counter
    ldx address_ptr_lo
    ldy address_ptr_hi
    ppu_update_loop:
      sty PPU_ADDR
      stx PPU_ADDR
      asl pad_2_axlr
      rol pad_2_keys
      lda sliver_pattren
      bcc ppu_update_skip_bit
        ora #$7c
      ppu_update_skip_bit:
      sta PPU_DATA
      txa
      clc
      adc #16
      tax
      bcc ppu_update_high_byte
        iny
      ppu_update_high_byte:
      dec update_counter
    bne ppu_update_loop
  paused:

  lda #$00
  sta PPU_SCROLL
  sta PPU_SCROLL
  lda #VBLANK_NMI|NT_2C00
  sta PPU_CTRL
  lda #BG_ON|OBJ_OFF
  sta PPU_MASK

  lda stop_state
  lsr
  ora pad_1_keys
__opcode_10:
  and #BUTTON_START
  bne paused_2
    ldx address_ptr_lo
    ldy address_ptr_hi
    inx
    txa
    and #%00000111
    bne not_tile_number_carry
      txa
      clc
      adc #(16*12)-8
      tax
      bcc not_tile_number_carry_high_byte
        iny
        cpy #>(16*12*20)
        bcc not_carry_to_next_plane
          ldy #0
          eor #%00001000
          tax
          bne skip_line_switch
            lda sliver_pattren
            eor #$82
            sta sliver_pattren
          skip_line_switch:
          lda stop_state
          bit __opcode_10+1   ;; bit #BUTTON_START
          beq dont_sync
            ;,;lda stop_state
            eor #$20
            sta stop_state
          dont_sync:
        not_carry_to_next_plane:
      not_tile_number_carry_high_byte:
    not_tile_number_carry:
    stx address_ptr_lo
    sty address_ptr_hi
    lda sliver_pattren
    eor #$01
    sta sliver_pattren
  paused_2:
  
  lda pad_1_keys
  and #BUTTON_SELECT
  beq not_program_exit
    lda pad_1_axlr
    and #$f0
    cmp #$10
    bne not_easter_egg
      lda pad_1_keys
      sbc #BUTTON_UP|BUTTON_LEFT|BUTTON_SELECT
      bne not_easter_egg2
        jsr easter_egg_entry
      not_easter_egg2:
    not_easter_egg:
    jmp ($fffc)
  not_program_exit:

  lda pad_1_keys
  eor #$ff
  and #BUTTON_A|BUTTON_B
  sta pad_1_keys

  lda stop_state
  and #$30
  ora pad_1_keys
  sta stop_state
jmp main_loop

do_ppu_pattren_load:
  sta temp+0
  attr_loop2:
    lda temp+0
    lsr
    ror temp+0
    txa
    bcc bit_cleared2
      eor #$ff
    bit_cleared2:
    sta PPU_DATA
    iny
  bne attr_loop2
rts
write_minus_y_number_of_blanks:
  lda #$ff
  chr_clear_inner_loop:
    sta PPU_DATA
    iny
  bne chr_clear_inner_loop
rts
draw_log_row:
  ldy #$fe
  sty PPU_DATA
  ldy #12
  ldx temp+0
  nt_loop2:
    stx PPU_DATA
    inx
    dey
  bne nt_loop2
  ldy #$fd
  sty PPU_DATA
rts
.endproc

.segment "RODATA"

font:
  .incbin "obj/button-logger-gfx.chr.bagel"

