.include "nes.inc"

.export cart_vectors

.import init_controller_music, button_logger_activity, easter_egg_entry
.exportzp temp
.exportzp nmi_count, tv_type, nmi_stash, rng_seed

.segment "INESHDR"
INES_MAPPER     = 2     ; 0 = NROM, 34 = BNROM
INES_PRG_COUNT  = 2     ; PRG ROM size in 16384 byte units
INES_CHR_COUNT  = 0     ; CHR ROM size in 8192 byte units
INES_MIRRORING  = 1     ; 0 = horizontal, 1 = vertical, 8 = four-screen
INES_SRAM       = 0     ; 1 = battery backed SRAM at $6000-7FFF
  .byte "NES",$1A       ; magic signature
  .byte INES_PRG_COUNT
  .byte INES_CHR_COUNT
  .byte ((INES_MAPPER & $0f) << 4) | (INES_SRAM << 1) | INES_MIRRORING
  .byte (INES_MAPPER & $f0)
  .byte 0,0,0,0,0,0,0,0     ; no other special features

.segment "ZEROPAGE": zeropage
  temp:                     .res 16
  nmi_count:                .res 1  ; must be at $10 for 240p overlay
  nmi_stash:                .res 1
  tv_type:                  .res 1
  _unused:                  .res 1
  rng_seed:                 .res 4

.segment "CODE"
reset_handlers:
  sei
  jmp init_controller_music
  sei
  jmp button_logger_activity
  sei
  jsr easter_egg_entry
  sei
  jmp ($fffc)

.segment "VECTORS"
cart_title:
; 16 byte zero filled UTF-8 title. with NMI embeded.
  .byte "Jroatch Toys."
NMI:
  inc nmi_count
IRQ:
rti
RST:
  sei
  ldx #$FF
  nop ;txs
  stx RST+2
jmp reset_handlers
cart_vectors:
  .addr NMI, RST, IRQ

