.include "nes.inc"

.importzp nmi_count, tv_type, nmi_stash
.exportzp ppustrings_ready, ppustrings_count
.exportzp ppu_var_scroll_x, ppu_var_scroll_y, ppu_var_ctrl, ppu_var_mask
.export ppu_oam, ppu_buffer

.import pads_read
.export ppu_vsync, ppu_set_addr, ppu_vsync_forced_blank, ppu_forced_blank
.export ppu_clear_oam, ppu_clear_oam_nocheck
.export ppustrings_init, ppustrings_init_with_callback, ppustrings_nmi_kernel
.export ppustrings_new_string, ppustrings_finish_string, ppustrings_write_run

.segment "ZEROPAGE": zeropage
  ppu_var_scroll:
    ppu_var_scroll_x:       .res 1
    ppu_var_scroll_y:       .res 1
  ppu_var_ctrl:             .res 1
  ppu_var_mask:             .res 1
  ppustrings_ready:         .res 1
    ; 00000000: do nothing, just run the callback.
    ; 0-------: DMA OAM, ppu strings, screen display/scroll, controller input.
    ; 1-------: screen display/scroll and controller input only.
  ppustrings_count:         .res 1
  ppustrings_buffer_ptr:    .res 1
  nmi_callback:             .res 2

.segment "OAMRAM"
  ppu_oam:                  .res 256

.segment "STACK"
  ppu_buffer:               .res 32*5

.segment "CODE"

.proc ppu_set_addr
  sta PPU_ADDR
  sty PPU_ADDR
rts
.endproc

.proc ppu_vsync
  lda nmi_count
  wait_loop:
    cmp nmi_count
  beq wait_loop
rts
.endproc

;;
; Moves all sprites starting at address Y (e.g, $04, $08, ..., $FC)
; below the visible area.
; Y is 0 at the end.
.proc ppu_clear_oam
  tya
  and #%11111100
  tay
    ; Round the address down to a multiple of 4 so that
    ; it won't freeze with invalid parameter input.
without_rounding:
  lda #$f8
    ; Any sprite Y value from $EF through $FF will work
  loop:
    sta ppu_oam, y
    iny
    iny
    iny
    iny
  bne loop
rts
.endproc

ppu_clear_oam_nocheck = ppu_clear_oam::without_rounding

;;
; Set the background color to black, scroll to the first nametable,
; reset the PPU write direction and enable NMI. $ff works because
; PPU_ADDR is mirrored after $4000 ($ff00 = $3f00) and any
; palette entry in the form %xxxx111x will always be black.
.proc ppu_vsync_forced_blank
  jsr ppu_vsync
just_blank:
  ldy #$00
  sty PPU_MASK
  lda #$ff
  jsr ppu_set_addr
  sta PPU_DATA
  sty PPU_SCROLL
  sty PPU_SCROLL
  lda #VBLANK_NMI|NT_2000
  sta PPU_CTRL
rts
.endproc

ppu_forced_blank = ppu_vsync_forced_blank::just_blank

.proc ppustrings_init
without_callback:
  lda #>(__rts_opcode-1)
  ldy #<(__rts_opcode-1)
with_callback_in_YA:
;  pha
;    lda #$40
;    sta ramvectors_nmi+0
;      ; lock NMI before changing the execution
;      ; path in ram vectors and callback.
;  pla
  sty nmi_callback+0
  sta nmi_callback+1
  lda #$00
  sta ppustrings_ready
  sta ppustrings_count
  sta ppustrings_buffer_ptr

;  lda #<ppustrings_nmi_kernel
;  sta ramvectors_nmi+1
;  lda #>ppustrings_nmi_kernel
;  sta ramvectors_nmi+2
;  lda #$4c
;  sta ramvectors_nmi+0
__rts_opcode:
rts
.endproc

ppustrings_init_with_callback = ppustrings_init::with_callback_in_YA

.proc ppustrings_nmi_kernel
  pha
;    lda #$40  ; rti opcode
;      ; Lock NMI, preventing re-entry.
;    sta ramvectors_nmi+0
    txa
    pha
      tya
      pha
        lda ppustrings_ready
        beq skip_all_ppu_updates
          eor PPU_STATUS
            ; Reset double write latch, but in a way that we can still
            ; use the N flag of ppustrings_ready. Because NMI starts at
            ; Vblank, this will always flip bit 7, and thus xor the N flag.
          bpl skip_dma_and_strings
            lda #BG_OFF|OBJ_OFF
            sta PPU_MASK
              ; just in case the ppu strings overflows vblank.

            ldy tv_type
            beq dma_not_first
              ;,; lda #0
              sta OAM_ADDR
              lda #>ppu_oam
              sta OAM_DMA
            dma_not_first:

            ldy ppustrings_count
            beq skip_ppu_strings
              tsx
              stx nmi_stash
              ldx #$ff
              txs
                ; pulling list items directly from the stack is risky
                ; even the NMI lock will not help a serious malfunction.
                next_string:
                  ldx #VBLANK_NMI
                  pla
                  bpl not_down
                    ldx #VBLANK_NMI|VRAM_DOWN
                  not_down:
                  sta PPU_ADDR
                  pla
                  sta PPU_ADDR
                  stx PPU_CTRL
                  pla
                  tax
                  bmi do_run
                  ;,; bpl do_lit
                  do_lit:
                    lit_loop:
                      pla
                      sta PPU_DATA
                      dex
                    bpl lit_loop
                    dey
                  bne next_string
                beq end_ppu_strings
                  do_run:
                    pla
                    run_loop:
                      sta PPU_DATA
                      inx
                    bne run_loop
                    dey
                  bne next_string
                ;,; beq end_ppu_strings
                end_ppu_strings:
              ldx nmi_stash
              txs
            skip_ppu_strings:
              ; Y = 0 here.

            lda tv_type
            bne dma_not_last
              ;,; lda #0
              sta OAM_ADDR
              lda #>ppu_oam
              sta OAM_DMA
            dma_not_last:
          skip_dma_and_strings:
            ; we need to make sure that there is
            ; an even amount of cycles between here
            ; and pads_read
          lda ppu_var_scroll_x
          sta PPU_SCROLL
          lda ppu_var_scroll_y
          sta PPU_SCROLL
          lda ppu_var_ctrl
          ora #VBLANK_NMI
          sta PPU_CTRL
          lda ppu_var_mask
          sta PPU_MASK
            ; total cycles = ((3 + 4) * 4) + 2 = 30
          jsr pads_read
            ; Y is not touched by pads_read
          ;,; ldy #$00
          sty ppustrings_ready
        skip_all_ppu_updates:
        cli
          ; As all updates for vblank are done,
          ; IRQs can be serviced now.
        jsr do_callback
      pla
      tay
    pla
    tax
    inc nmi_count
;    lda #$4c  ; jmp opcode
;    sta ramvectors_nmi+0
;      ; unlock NMI
  pla
rts
do_callback:
  lda nmi_callback+1
  pha
  lda nmi_callback+0
  pha
rts
.endproc

;;
; Y, A:  PPU_ADDR
.proc ppustrings_new_string
  ldx ppustrings_buffer_ptr
  sta ppu_buffer+0, x
  tya
  sta ppu_buffer+1, x
  inx
  inx
  inx
rts
.endproc

;;
; X: buffer pointer, pointing to after the last byte.
.proc ppustrings_finish_string
  txa
  sec
  sbc #4
  sec
  sbc ppustrings_buffer_ptr
  bmi invalid_length
    ldy ppustrings_buffer_ptr
    sta ppu_buffer+2, y
    stx ppustrings_buffer_ptr
    inc ppustrings_count
  invalid_length:
rts
.endproc

;;
; A: run value
; Y: run length
.proc ppustrings_write_run
  ldx ppustrings_buffer_ptr
  sta ppu_buffer+3, x
  dey
  tya
  eor #$ff
  sta ppu_buffer+2, x
  bpl invalid_length
    inx
    inx
    inx
    inx
    stx ppustrings_buffer_ptr
    inc ppustrings_count
  invalid_length:
rts
.endproc
