#!/usr/bin/env python3

import sys
from PIL import Image

PAGE_SIZE = 8192

for fn in sys.argv[1:]:
    pages = []
    block = b''
    block_number = 0
    with open(fn, "rb") as f:
        potential_header = f.read(16)
        if potential_header[0:4] != b'NES\x1a':
            block = potential_header + f.read(PAGE_SIZE-16)
        else:
            block = f.read(PAGE_SIZE)
        
        while len(block) > 0:
            page_img = Image.new("LA", (128, 256), (0,0))
            page_buf = page_img.load()
            for y in range(256):
                for x in range(128):
                    ty, py, tx, px = y // 8, y % 8, x // 8, x % 8 
                    i = ty * 256 + tx * 16 + py
                    if i+8 < len(block):
                        bit1 = (block[i] >> (7 - px)) & 1
                        bit2 = (block[i+8] >> (7 - px)) & 1
                        page_buf[x, y] = ((bit2 * 2 + bit1) * 0x55, 255)
                    else:
                        page_buf[x, y] = (0, 0)
            block_number += 1
            pages.append(page_img)
            block = f.read(PAGE_SIZE)
        output_img_height = (((len(pages)-1)//4)+1)*256
        if len(pages) <= 4:
            output_img = Image.new("LA", (len(pages)*128, 256), (0,0))
        else:
            output_img = Image.new("LA", (512, output_img_height), (0,0))
        for page_number in range(len(pages)):
            output_img.paste(pages[page_number], ((page_number%4)*128,(page_number//4)*256))
        output_img.save(fn + ".png")
